<?php
/***************************************************************
 *  Copyright notice
 *  (c) 1999-2006 Kasper Skaarhoj (kasperYYYY@typo3.com)
 *  All rights reserved
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *  A copy is found in the textfile GPL.txt and important notices to the license
 *  from the author is found in LICENSE.txt distributed with these scripts.
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  This copyright notice MUST APPEAR in all copies of the script!
 ***************************************************************/
/**
 * Contains a class with "Page functions" mainly for the frontend
 * $Id: class.t3lib_page.php 1992 2007-02-04 22:52:47Z mundaun $
 * Revised for TYPO3 3.6 2/2003 by Kasper Skaarhoj
 * XHTML-trans compliant
 *
 * @author    Kasper Skaarhoj <kasperYYYY@typo3.com>
 */

/**
 * [CLASS/FUNCTION INDEX of SCRIPT]
 *  109: class t3lib_pageSelect
 *  134:     function init($show_hidden)
 *              SECTION: Selecting page records
 *  184:     function getPage($uid, $disableGroupAccessCheck=FALSE)
 *  200:     function getPage_noCheck($uid)
 *  216:     function getFirstWebPage($uid)
 *  234:     function getPageIdFromAlias($alias)
 *  250:     function getPageOverlay($pageInput,$lUid=-1)
 *  314:     function getRecordOverlay($table,$row,$sys_language_content,$OLmode='')
 *              SECTION: Page related: Menu, Domain record, Root line
 *  413:     function getMenu($uid,$fields='*',$sortField='sorting',$addWhere='',$checkShortcuts=1)
 *  471:     function getDomainStartPage($domain, $path='',$request_uri='')
 *  519:     function getRootLine($uid, $MP='', $ignoreMPerrors=FALSE)
 *  640:     function getPathFromRootline($rl,$len=20)
 *  661:     function getExtURL($pagerow,$disable=0)
 *  685:     function getMountPointInfo($pageId, $pageRec=FALSE, $prevMountPids=array(), $firstPageUid=0)
 *              SECTION: Selecting records in general
 *  762:     function checkRecord($table,$uid,$checkPage=0)
 *  797:     function getRawRecord($table,$uid,$fields='*',$noWSOL=FALSE)
 *  823:     function getRecordsByField($theTable,$theField,$theValue,$whereClause='',$groupBy='',$orderBy='',$limit='')
 *              SECTION: Caching and standard clauses
 *  875:     function getHash($hash,$expTime=0)
 *  898:     function storeHash($hash,$data,$ident)
 *  916:     function deleteClause($table)
 *  936:     function enableFields($table,$show_hidden=-1,$ignore_array=array(),$noVersionPreview=FALSE)
 * 1008:     function getMultipleGroupsWhereClause($field, $table)
 *              SECTION: Versioning Preview
 * 1055:     function fixVersioningPid($table,&$rr)
 * 1096:     function versionOL($table,&$row)
 * 1151:     function getWorkspaceVersionOfRecord($workspace, $table, $uid, $fields='*')
 * TOTAL FUNCTIONS: 24
 * (This index is automatically created/updated by the extension "extdeveval")
 */
class ux_t3lib_page extends \TYPO3\CMS\Frontend\Page\PageRepository {

    /**
     * function that checks which page record overlay language should be used, according to the given fallbackorder
     *
     * @param integer $page_id       pid which is checked for translations
     * @param string  $fallBackOrder comma seperated list of language uids (integer). the languageUid to whith the fallback order
     * @return integer  id of the first found languageuid in the fallbackorder where a translation for $pid exists.
     **/
    function getLanguageOverlayUidForPage($page_id, $fallBackOrder) {
        $lUids = \TYPO3\CMS\Core\Utility\GeneralUtility::intExplode(',', $fallBackOrder);
        foreach ($lUids as $lUid) {
            // Selecting overlay record:
            $res = $GLOBALS['TYPO3_DB']->exec_SELECTquery('*', 'pages_language_overlay', 'pid=' . intval($page_id) . '
								AND sys_language_uid=' . intval($lUid) . $this->enableFields('pages_language_overlay'), '', '', '1');
            $row = $GLOBALS['TYPO3_DB']->sql_fetch_assoc($res);
            $this->versionOL('pages_language_overlay', $row);
            if (is_array($row)) {
                return $lUid;
            }
        }

        return 0;
    }

    /**
     * Returns the relevant page overlay record fields
     *
     * @param    mixed   $pageInput If $pageInput is an integer, it's the pid of the pageOverlay record and thus the page overlay record is returned. If $pageInput is an array, it's a page-record and based on this page record the language record is found and OVERLAYED before the page record is returned.
     * @param    integer $lUid      Language UID if you want to set an alternative value to $this->sys_language_uid which is default. Should be >=0
     * @return    array        Page row which is overlayed with language_overlay record (or the overlay record alone)
     */
    function getPageOverlay($pageInput, $lUid = -1) {
        //get page_id
        if (is_array($pageInput)) {
            $page_id = $pageInput['uid'];    // Was the whole record
        }
        else {
            $page_id = $pageInput;    // Was the id
        }
        // Initialize:
        if ($lUid < 0) {
            $lUid = $this->sys_language_uid;
            //if  with sys_language_overlayCheckOrder, check for other Overlay language which should be used instead
            if ($page_id && $lUid && is_object($GLOBALS['TSFE']) && $GLOBALS['TSFE']->sys_language_overlayCheckOrder) {
                $lUid = $this->getLanguageOverlayUidForPage($page_id, $GLOBALS['TSFE']->sys_language_overlayCheckOrder);
            }
        }
        $row = null;
        // If language UID is different from zero, do overlay:
        if ($lUid) {
            $fieldArr = explode(',', $GLOBALS['TYPO3_CONF_VARS']['FE']['pageOverlayFields']);
            if (is_array($pageInput)) {
                $fieldArr = array_intersect($fieldArr, array_keys($pageInput));        // Make sure that only fields which exist in the incoming record are overlaid!
            }

            if (count($fieldArr)) {
                /*
                    NOTE to enabledFields('pages_language_overlay'):
                    Currently the showHiddenRecords of TSFE set will allow pages_language_overlay records to be selected as they are child-records of a page.
                    However you may argue that the showHiddenField flag should determine this. But that's not how it's done right now.
                */

                // Selecting overlay record:
                $res = $GLOBALS['TYPO3_DB']->exec_SELECTquery(implode(',', $fieldArr), 'pages_language_overlay', 'pid=' . intval($page_id) . '
								AND sys_language_uid=' . intval($lUid) . $this->enableFields('pages_language_overlay'), '', '', '1');
                $row = $GLOBALS['TYPO3_DB']->sql_fetch_assoc($res);
                $this->versionOL('pages_language_overlay', $row);

                if (is_array($row)) {
                    $row['_PAGES_OVERLAY']      = true;
                    $row['_PAGES_OVERLAY_LUID'] = $lUid;
                    // Unset vital fields that are NOT allowed to be overlaid:
                    unset($row['uid']);
                    unset($row['pid']);
                }
            }
        }

        // Create output:
        if (is_array($pageInput)) {
            return is_array($row) ? array_merge($pageInput, $row) : $pageInput;    // If the input was an array, simply overlay the newfound array and return...
        }
        else {
            return is_array($row) ? $row : array();    // always an array in return
        }
    }
}