<?php
/***************************************************************
 *  Copyright notice
 *  (c) 1999-2007 Kasper Skaarhoj (kasperYYYY@typo3.com)
 *  All rights reserved
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *  A copy is found in the textfile GPL.txt and important notices to the license
 *  from the author is found in LICENSE.txt distributed with these scripts.
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  This copyright notice MUST APPEAR in all copies of the script!
 ***************************************************************/
/**
 * Class for the built TypoScript based Front End
 * This class has a lot of functions and internal variable which are use from index_ts.php.
 * The class is instantiated as $GLOBALS['TSFE'] in index_ts.php.
 * The use of this class should be inspired by the order of function calls as found in index_ts.php.
 * $Id: class.tslib_fe.php 2065 2007-02-20 22:09:19Z ohader $
 * Revised for TYPO3 3.6 June/2003 by Kasper Skaarhoj
 * XHTML compliant
 *
 * @author    Kasper Skaarhoj <kasperYYYY@typo3.com>
 */

/**
 * Main frontend class, instantiated in the index_ts.php script as the global object TSFE
 *
 * @author     Kasper Skaarhoj <kasperYYYY@typo3.com>
 * @package    TYPO3
 * @subpackage tslib
 */
class ux_tslib_fe extends \TYPO3\CMS\Frontend\Controller\TypoScriptFrontendController {

    /**
     * Setting the language key that'll be used by the current page.
     * In this function it should be checked, 1) that this language exists, 2) that a page_overlay_record exists, .. and if not the default language, 0 (zero), should be set.
     *
     * @return    void
     */
    function settingLanguage() {
        // Get values from TypoScript:
        $this->sys_language_uid = $this->sys_language_content = intval($this->config['config']['sys_language_uid']);
        list($this->sys_language_mode, $sys_language_content) = \TYPO3\CMS\Core\Utility\GeneralUtility::trimExplode(';', $this->config['config']['sys_language_mode']);
        $this->sys_language_contentOL = $this->config['config']['sys_language_overlay'];
        // If sys_language_uid is set to another language than default:
        if ($this->sys_language_uid > 0) {
            //if other language requested we need to check to set $this->sys_language_overlayCheckOrder (used for menu generation...)
            if ($this->config['config']['sys_language_fallBackOrder']) {
                if ($this->config['config']['sys_language_fallBackOrder']) {
                    $this->sys_language_overlayCheckOrder = $this->sys_language_uid . ',' . $this->config['config']['sys_language_fallBackOrder'];    //used for detecting overlays for page records (e.g. menu ...)
                }
            }
            // Request the overlay record for the sys_language_uid:
            $olRec = $this->sys_page->getPageOverlay($this->id, $this->sys_language_uid);
            if (!count($olRec)) {

                // If no OL record exists and a foreign language is asked for...
                if ($this->sys_language_uid) {

                    // If requested translation is not available:
                    if (\TYPO3\CMS\Core\Utility\GeneralUtility::hideIfNotTranslated($this->page['l18n_cfg'])) {
                        $this->pageNotFoundAndExit('Page is not available in the requested language.');
                    }
                    else {
                        switch ((string) $this->sys_language_mode) {
                            case 'strict':
                                $this->pageNotFoundAndExit('Page is not available in the requested language (strict).');
                                break;
                            case 'content_fallback':
                                // is alternative fallback Order specified for content_fallback? use this instead sys_language_overlayCheckOrder
                                $fallBackOrder = $sys_language_content;
                                if ($fallBackOrderContent) {
                                    $overlayLanguageUidForCurrentPage = $this->sys_page->getLanguageOverlayUidForPage($this->id, $fallBackOrderContent);
                                }
                                else {
                                    $overlayLanguageUidForCurrentPage = $this->sys_page->getLanguageOverlayUidForPage($this->id, $this->sys_language_overlayCheckOrder);
                                }
                                $this->sys_language_content = $overlayLanguageUidForCurrentPage;
                                // be aware of alternative Fallback language UIds for this page and load new overlayed record for this page
                                if ($overlayLanguageUidForCurrentPage > 0) {
                                    $this->page = $this->sys_page->getPageOverlay($this->page, $overlayLanguageUidForCurrentPage);
                                }
                                break;
                            case 'ignore':
                                $this->sys_language_content = $this->sys_language_uid;
                                break;
                            default:
                                // Default is that everything defaults to the default language...
                                $this->sys_language_uid = $this->sys_language_content = 0;
                                break;
                        }
                    }
                }
            }
            else {
                // Setting sys_language if an overlay record was found (which it is only if a language is used)
                $this->page = $this->sys_page->getPageOverlay($this->page, $this->sys_language_uid);
            }
        }

        // Setting sys_language_uid inside sys-page:
        $this->sys_page->sys_language_uid = $this->sys_language_uid;

        // If default translation is not available:
        if ((!$this->sys_language_uid || !$this->sys_language_content) && $this->page['l18n_cfg'] & 1) {
            $this->pageNotFoundAndExit('Page is not available in default language.');
        }

        // Updating content of the two rootLines IF the language key is set!
        if ($this->sys_language_uid && is_array($this->tmpl->rootLine)) {
            reset($this->tmpl->rootLine);
            while (list($rLk) = each($this->tmpl->rootLine)) {
                $this->tmpl->rootLine[$rLk] = $this->sys_page->getPageOverlay($this->tmpl->rootLine[$rLk]);
            }
        }
        if ($this->sys_language_uid && is_array($this->rootLine)) {
            reset($this->rootLine);
            while (list($rLk) = each($this->rootLine)) {
                $this->rootLine[$rLk] = $this->sys_page->getPageOverlay($this->rootLine[$rLk]);
            }
        }

        // Finding the ISO code:
        if (t3lib_extMgm::isLoaded('static_info_tables') && $this->sys_language_content) {    // using sys_language_content because the ISO code only (currently) affect content selection from FlexForms - which should follow "sys_language_content"
            $sys_language_row = $this->sys_page->getRawRecord('sys_language', $this->sys_language_content, 'static_lang_isocode');
            if (is_array($sys_language_row) && $sys_language_row['static_lang_isocode']) {
                $stLrow                     = $this->sys_page->getRawRecord('static_languages', $sys_language_row['static_lang_isocode'], 'lg_iso_2');
                $this->sys_language_isocode = $stLrow['lg_iso_2'];
            }
        }

        // Setting softMergeIfNotBlank:
        $table_fields = \TYPO3\CMS\Core\Utility\GeneralUtility::trimExplode(',', $this->config['config']['sys_language_softMergeIfNotBlank'], 1);
        foreach ($table_fields as $TF) {
            list($tN, $fN) = explode(':', $TF);
            $this->TCAcachedExtras[$tN]['l10n_mode'][$fN] = 'mergeIfNotBlank';
        }

        // Setting softExclude:
        $table_fields = \TYPO3\CMS\Core\Utility\GeneralUtility::trimExplode(',', $this->config['config']['sys_language_softExclude'], 1);
        foreach ($table_fields as $TF) {
            list($tN, $fN) = explode(':', $TF);
            $this->TCAcachedExtras[$tN]['l10n_mode'][$fN] = 'exclude';
        }
    }
}
