<?php

/**
 * Class ux_tslib_menu
 *
 * @author Sven Reuter (sreuter)
 */
class ux_tslib_menu extends \TYPO3\CMS\Frontend\ContentObject\Menu\AbstractMenuContentObject {

    /**
     * Checks if a page is OK to include in the final menu item array. Pages can be excluded if the doktype is wrong, if they are hidden in navigation, have a uid in the list of banned uids etc.
     *
     * @param    array   $data        Array of menu items
     * @param    array   $banUidArray Array of page uids which are to be excluded
     * @param    boolean $spacer      If set, then the page is a spacer.
     * @return    boolean        Returns true if the page can be safely included.
     */
    function filterMenuPages(&$data, $banUidArray, $spacer) {

        if ($data['_SAFE']) {
            return true;
        }

        $uid = $data['uid'];
        if ($this->mconf['SPC'] || !$spacer) {    // If the spacer-function is not enabled, spacers will not enter the $menuArr
            if (!\TYPO3\CMS\Core\Utility\GeneralUtility::inList($this->doktypeExcludeList, $data['doktype'])) {        // Page may not be 'not_in_menu' or 'Backend User Section'
                if (!$data['nav_hide'] || $this->conf['includeNotInMenu']) {    // Not hidden in navigation
                    if (!\TYPO3\CMS\Core\Utility\GeneralUtility::inArray($banUidArray, $uid)) {    // not in banned uid's

                        // Checks if the default language version can be shown:
                        // Block page is set, if l18n_cfg allows plus: 1) Either default language or 2) another language but NO overlay record set for page!
                        $blockPage = $data['l18n_cfg'] & 1 && (!$GLOBALS['TSFE']->sys_language_uid || ($GLOBALS['TSFE']->sys_language_uid && !$data['_PAGES_OVERLAY']));
                        if (!$blockPage) {

                            // Checking if a page should be shown in the menu depending on whether a translation exists:
                            $tok = true;
                            if ($GLOBALS['TSFE']->sys_language_uid && \TYPO3\CMS\Core\Utility\GeneralUtility::hideIfNotTranslated($data['l18n_cfg'])) {    // There is an alternative language active AND the current page requires a translation:
                                if (!$data['_PAGES_OVERLAY'] || ($data['_PAGES_OVERLAY'] && $data['_PAGES_OVERLAY_LUID'] != $GLOBALS['TSFE']->sys_language_uid)) {
                                    $tok = false;
                                }
                            }

                            // Continue if token is true:
                            if ($tok) {

                                // Checking if "&L" should be modified so links to non-accessible pages will not happen.
                                if ($this->conf['protectLvar']) {
                                    $languageUid = intval($GLOBALS['TSFE']->config['config']['sys_language_uid']);
                                    if ($languageUid && ($this->conf['protectLvar'] == 'all' || \TYPO3\CMS\Core\Utility\GeneralUtility::hideIfNotTranslated($data['l18n_cfg']))) {
                                        $olRec = $GLOBALS['TSFE']->sys_page->getPageOverlay($data['uid'], $languageUid);
                                        if (!count($olRec)) {
                                            // If no pages_language_overlay record then page can NOT be accessed in the language pointed to by "&L" and therefore we protect the link by setting "&L=0"
                                            $data['_ADD_GETVARS'] .= '&L=0';
                                        }
                                    }
                                }

                                return true;
                            }
                        }
                    }
                }
            }
        }

        return false;
    }
}